@@ -1,5 +1,5 @@
 /*
- * Hedgewars, a worms-like game
+ * Hedgewars, a free turn based strategy game
  * Copyright (c) 2006, 2007 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -36,11 +36,17 @@ public:
 	QRect vid_Resolution();
 	bool vid_Fullscreen();
 	bool isSoundEnabled();
+	bool isMusicEnabled();
 	bool isShowFPSEnabled();
 	bool isAltDamageEnabled();
 	quint8 timerInterval();
 	quint8 bitDepth();
 	QString netNick();
+	bool isFrontendFullscreen() const;
+	void resizeToConfigValues();
+
+ signals:
+	void frontendFullscreen(bool value);
 
 public slots:
 	void SaveOptions();
