@@ -1,5 +1,5 @@
 /*
- * Hedgewars, a worms-like game
+ * Hedgewars, a free turn based strategy game
  * Copyright (c) 2005-2008 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -30,6 +30,7 @@ extern QDir * cfgdir;
 extern QDir * datadir;
 
 extern QStringList * Themes;
+extern QStringList * mapList;
 
 extern QString * cDefaultAmmoStore;
 
@@ -42,3 +43,5 @@ extern QColor * color6;
 
 extern QString * netHost;
 extern quint16 netPort;
+
+extern bool haveServer;
