@@ -1,5 +1,5 @@
 /*
- * Hedgewars, a worms-like game
+ * Hedgewars, a free turn based strategy game
  * Copyright (c) 2005-2008 Andrey Korotaev <unC0Rr@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,11 @@
 #include <QRadioButton>
 #include <QSpinBox>
 #include <QCloseEvent>
+#include <QCheckBox>
+#include <QTextBrowser>
+#include <QAction>
+#include <QTimer>
+#include <QScrollBar>
 
 #include "hwform.h"
 #include "game.h"
@@ -40,7 +45,6 @@
 #include "gamecfgwidget.h"
 #include "netserverslist.h"
 #include "netudpserver.h"
-#include "netwwwserver.h"
 #include "chatwidget.h"
 #include "playrecordpage.h"
 #include "input_ip.h"
@@ -49,16 +53,21 @@ HWForm::HWForm(QWidget *parent)
   : QMainWindow(parent), pnetserver(0), pRegisterServer(0), editedTeam(0), hwnet(0)
 {
 	ui.setupUi(this);
+
+	CustomizePalettes();
+
+	ui.pageOptions->CBResolution->addItems(sdli.getResolutions());
+
 	config = new GameUIConfig(this, cfgdir->absolutePath() + "/hedgewars.ini");
 
 	UpdateTeamsLists();
 	UpdateWeapons();
 
+	connect(config, SIGNAL(frontendFullscreen(bool)), this, SLOT(onFrontendFullscreen(bool)));
+	onFrontendFullscreen(config->isFrontendFullscreen());
+
 	connect(ui.pageMain->BtnSinglePlayer,	SIGNAL(clicked()),	this, SLOT(GoToSinglePlayer()));
 	connect(ui.pageMain->BtnSetup,	SIGNAL(clicked()),	this, SLOT(GoToSetup()));
-	connect(ui.pageMain->BtnMultiplayer,	SIGNAL(clicked()),	this, SLOT(GoToMultiplayer()));
-	connect(ui.pageMain->BtnLoad,	SIGNAL(clicked()),	this, SLOT(GoToSaves()));
-	connect(ui.pageMain->BtnDemos,	SIGNAL(clicked()),	this, SLOT(GoToDemos()));
 	connect(ui.pageMain->BtnNet,	SIGNAL(clicked()),	this, SLOT(GoToNet()));
 	connect(ui.pageMain->BtnInfo,	SIGNAL(clicked()),	this, SLOT(GoToInfo()));
 	connect(ui.pageMain->BtnExit, SIGNAL(pressed()), this, SLOT(btnExitPressed()));
@@ -88,26 +97,31 @@ HWForm::HWForm(QWidget *parent)
 	connect(ui.pageSelectWeapon->pWeapons,       SIGNAL(weaponsChanged()), this, SLOT(UpdateWeapons()));
 	connect(ui.pageNetGame->pGameCFG,       SIGNAL(newWeaponsName(const QString&)), this, SLOT(NetWeaponNameChanged(const QString&)));
 
-	connect(ui.pageNet->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageNet->BtnSpecifyServer,	SIGNAL(clicked()),	this, SLOT(NetConnect()));
-	connect(ui.pageNet->BtnNetSvrStart,	SIGNAL(clicked()),	this, SLOT(GoToNetServer()));
-	connect(ui.pageNet,	SIGNAL(connectClicked(const QString &, quint16)),	this, SLOT(NetConnectServer(const QString &, quint16)));
+	connect(ui.pageNet->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+	connect(ui.pageNet->BtnSpecifyServer, SIGNAL(clicked()), this, SLOT(NetConnect()));
+	connect(ui.pageNet->BtnOfficialServer, SIGNAL(clicked()), this, SLOT(NetConnectOfficialServer()));
+	connect(ui.pageNet->BtnNetSvrStart, SIGNAL(clicked()), this, SLOT(GoToNetServer()));
+	connect(ui.pageNet, SIGNAL(connectClicked(const QString &, quint16)), this, SLOT(NetConnectServer(const QString &, quint16)));
 	
 	connect(ui.pageNetServer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 	connect(ui.pageNetServer->BtnStart,	SIGNAL(clicked()),	this, SLOT(NetStartServer()));
 
 	connect(ui.pageNetGame->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
-	connect(ui.pageNetGame->BtnGo,	SIGNAL(clicked()),	this, SLOT(NetStartGame()));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(setEnabledGameStart(bool)),
 		ui.pageNetGame->BtnGo, SLOT(setEnabled(bool)));
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(SetupClicked()), this, SLOT(IntermediateSetup()));
 
-	connect(ui.pageInfo->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageRoomsList->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
-	connect(ui.pageGameStats->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
+	connect(ui.pageInfo->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
+
+	connect(ui.pageGameStats->BtnBack, SIGNAL(clicked()), this, SLOT(GoBack()));
 
 	connect(ui.pageSinglePlayer->BtnSimpleGamePage,	SIGNAL(clicked()),	this, SLOT(SimpleGame()));
 	connect(ui.pageSinglePlayer->BtnTrainPage,	SIGNAL(clicked()),	this, SLOT(GoToTraining()));
+	connect(ui.pageSinglePlayer->BtnMultiplayer,	SIGNAL(clicked()),	this, SLOT(GoToMultiplayer()));
+	connect(ui.pageSinglePlayer->BtnLoad,	SIGNAL(clicked()),	this, SLOT(GoToSaves()));
+	connect(ui.pageSinglePlayer->BtnDemos,	SIGNAL(clicked()),	this, SLOT(GoToDemos()));
 	connect(ui.pageSinglePlayer->BtnBack,	SIGNAL(clicked()),	this, SLOT(GoBack()));
 
 	connect(ui.pageTraining->BtnStartTrain,	SIGNAL(clicked()),	this, SLOT(StartTraining()));
@@ -127,28 +141,53 @@ HWForm::HWForm(QWidget *parent)
 	GoToPage(ID_PAGE_MAIN);
 }
 
+void HWForm::onFrontendFullscreen(bool value)
+{
+  if (value)
+    setWindowState(windowState() | Qt::WindowFullScreen);
+  else {
+    setWindowState(windowState() & !Qt::WindowFullScreen);
+  }
+}
+
+void HWForm::CustomizePalettes()
+{
+	QList<QScrollBar *> allSBars = findChildren<QScrollBar *>();
+	QPalette pal = palette();
+	pal.setColor(QPalette::WindowText, QColor(0xff, 0xcc, 0x00));
+	pal.setColor(QPalette::Button, QColor(0x00, 0x35, 0x1d));
+	pal.setColor(QPalette::Base, QColor(0x00, 0x35, 0x1d));
+	pal.setColor(QPalette::Window, QColor(0x00, 0x00, 0x00));
+
+	for (int i = 0; i < allSBars.size(); ++i)
+		allSBars.at(i)->setPalette(pal);
+}
+
 void HWForm::UpdateWeapons()
 {
-  // FIXME: rewrite this with boost (or TR1/0x)
-  QVector<QComboBox*> combos;
-  combos.push_back(ui.pageOptions->WeaponsName);
-  combos.push_back(ui.pageMultiplayer->gameCFG->WeaponsName);
-  combos.push_back(ui.pageNetGame->pGameCFG->WeaponsName);
+	// FIXME: rewrite this with boost (or TR1/0x)
+	QVector<QComboBox*> combos;
+	combos.push_back(ui.pageOptions->WeaponsName);
+	combos.push_back(ui.pageMultiplayer->gameCFG->WeaponsName);
+	combos.push_back(ui.pageNetGame->pGameCFG->WeaponsName);
 
-  for(QVector<QComboBox*>::iterator it=combos.begin(); it!=combos.end(); ++it) {
-    (*it)->clear();
-    (*it)->addItems(ui.pageSelectWeapon->pWeapons->getWeaponNames());
-    int pos=(*it)->findText("Default");
-    if (pos!=-1) {
-      (*it)->setCurrentIndex(pos);
-    }
-  }
+	for(QVector<QComboBox*>::iterator it=combos.begin(); it!=combos.end(); ++it) {
+		(*it)->clear();
+		(*it)->addItems(ui.pageSelectWeapon->pWeapons->getWeaponNames());
+		int pos=(*it)->findText("Default");
+		if (pos!=-1) {
+			(*it)->setCurrentIndex(pos);
+		}
+	}
 }
 
 void HWForm::NetWeaponNameChanged(const QString& name)
 {
-  QString ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
-  hwnet->onWeaponsNameChanged(name, ammo);
+	QString ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(
+			ui.pageNetGame->pGameCFG->WeaponsName->currentText()
+			);
+
+	hwnet->onWeaponsNameChanged(name, ammo);
 }
 
 void HWForm::UpdateTeamsLists(const QStringList* editable_teams)
@@ -240,25 +279,28 @@ void HWForm::GoToNetServer()
 
 void HWForm::OnPageShown(quint8 id, quint8 lastid)
 {
-	if (id == ID_PAGE_MULTIPLAYER || id == ID_PAGE_NETCFG) {
-		QStringList tmNames=config->GetTeamsList();
+	if (id == ID_PAGE_MULTIPLAYER || id == ID_PAGE_NETGAME) {
+		QStringList tmNames = config->GetTeamsList();
 		TeamSelWidget* curTeamSelWidget;
+		
 		if(id == ID_PAGE_MULTIPLAYER) {
-		  curTeamSelWidget=ui.pageMultiplayer->teamsSelect;
+		  curTeamSelWidget = ui.pageMultiplayer->teamsSelect;
 		} else {
-		  curTeamSelWidget=ui.pageNetGame->pNetTeamsWidget;
+		  curTeamSelWidget = ui.pageNetGame->pNetTeamsWidget;
 		}
+		
 		QList<HWTeam> teamsList;
-		for(QStringList::iterator it=tmNames.begin(); it!=tmNames.end(); it++) {
+		for(QStringList::iterator it = tmNames.begin(); it != tmNames.end(); it++) {
 		  HWTeam team(*it);
 		  team.LoadFromFile();
 		  teamsList.push_back(team);
 		}
-		if(lastid==ID_PAGE_SETUP) { // _TEAM
+		
+		if(lastid == ID_PAGE_SETUP) { // _TEAM
 		  if (editedTeam) {
 		    curTeamSelWidget->addTeam(*editedTeam);
 		  }
-		} else {
+		} else if(lastid != ID_PAGE_GAMESTATS && lastid != ID_PAGE_INGAME) {
 		  curTeamSelWidget->resetPlayingTeams(teamsList);
 		}
 	}
@@ -266,7 +308,7 @@ void HWForm::OnPageShown(quint8 id, quint8 lastid)
 
 void HWForm::GoToPage(quint8 id)
 {
-	quint8 lastid=ui.Pages->currentIndex();
+	quint8 lastid = ui.Pages->currentIndex();
 	PagesStack.push(ui.Pages->currentIndex());
 	OnPageShown(id, lastid);
 	ui.Pages->setCurrentIndex(id);
@@ -274,12 +316,17 @@ void HWForm::GoToPage(quint8 id)
 
 void HWForm::GoBack()
 {
-	if (!PagesStack.isEmpty() && PagesStack.top() == ID_PAGE_NET) {
-	  if(hwnet || pnetserver) NetDisconnect();
-	}
 	quint8 id = PagesStack.isEmpty() ? ID_PAGE_MAIN : PagesStack.pop();
-	OnPageShown(id, ui.Pages->currentIndex());
+	quint8 curid = ui.Pages->currentIndex();
 	ui.Pages->setCurrentIndex(id);
+	OnPageShown(id, curid);
+	
+	if (id == ID_PAGE_ROOMSLIST || id == ID_PAGE_NETSERVER) {
+		GoBack();
+	}
+	if (id == ID_PAGE_NET) {
+		if(hwnet || pnetserver) NetDisconnect();
+	}
 }
 
 void HWForm::btnExitPressed()
@@ -375,7 +422,12 @@ void HWForm::PlayDemo()
 
 void HWForm::NetConnectServer(const QString & host, quint16 port)
 {
-  _NetConnect(host, port, ui.pageOptions->editNetNick->text());
+	_NetConnect(host, port, ui.pageOptions->editNetNick->text());
+}
+
+void HWForm::NetConnectOfficialServer()
+{
+	NetConnectServer("netserver.hedgewars.org", 46631);
 }
 
 void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString & nick)
@@ -385,21 +437,47 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 		delete hwnet;
 		hwnet=0;
 	}
+	
 	ui.pageNetGame->pChatWidget->clear();
+	
 	hwnet = new HWNewNet(config, ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget);
 
 	connect(hwnet, SIGNAL(AskForRunGame()), this, SLOT(CreateNetGame()));
+	connect(hwnet, SIGNAL(Connected()), this, SLOT(NetConnected()));
 	connect(hwnet, SIGNAL(EnteredGame()), this, SLOT(NetGameEnter()));
 	connect(hwnet, SIGNAL(AddNetTeam(const HWTeam&)), this, SLOT(AddNetTeam(const HWTeam&)));
 
-	connect(hwnet, SIGNAL(chatStringFromNet(const QStringList&)),
-		ui.pageNetGame->pChatWidget, SLOT(onChatStringFromNet(const QStringList&)));
+	connect(hwnet, SIGNAL(roomsList(const QStringList&)),
+		ui.pageRoomsList, SLOT(setRoomsList(const QStringList&)));
+	connect(hwnet, SIGNAL(serverMessage(const QString&)),
+		ui.pageRoomsList->serverMessage, SLOT(setText(const QString&)));
+	
+	connect(ui.pageRoomsList, SIGNAL(askForCreateRoom(const QString &)),
+		hwnet, SLOT(CreateRoom(const QString&)));
+	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
+		hwnet, SLOT(JoinRoom(const QString&)));
+	connect(ui.pageRoomsList, SIGNAL(askForCreateRoom(const QString &)),
+		this, SLOT(NetGameMaster()));
+	connect(ui.pageRoomsList, SIGNAL(askForJoinRoom(const QString &)),
+		this, SLOT(NetGameSlave()));
+	connect(ui.pageRoomsList, SIGNAL(askForRoomList()),
+		hwnet, SLOT(askRoomsList()));
+
+	connect(hwnet, SIGNAL(chatStringFromNet(const QString&)),
+		ui.pageNetGame->pChatWidget, SLOT(onChatString(const QString&)));
+	connect(hwnet, SIGNAL(setReadyStatus(const QString &, bool)),
+		ui.pageNetGame->pChatWidget, SLOT(setReadyStatus(const QString &, bool)));
+	connect(hwnet, SIGNAL(chatStringFromMe(const QString&)),
+		ui.pageNetGame->pChatWidget, SLOT(onChatString(const QString&)));
 	connect(ui.pageNetGame->pChatWidget, SIGNAL(chatLine(const QString&)),
 		hwnet, SLOT(chatLineToNet(const QString&)));
+	connect(ui.pageNetGame->pChatWidget, SIGNAL(kick(const QString&)),
+		hwnet, SLOT(kickPlayer(const QString&)));
 	connect(hwnet, SIGNAL(nickAdded(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(nickAdded(const QString&)));
 	connect(hwnet, SIGNAL(nickRemoved(const QString&)),
 		ui.pageNetGame->pChatWidget, SLOT(nickRemoved(const QString&)));
+	connect(ui.pageNetGame->BtnGo, SIGNAL(clicked()), hwnet, SLOT(ToggleReady()));
 
 	connect(ui.pageNetGame->pNetTeamsWidget, SIGNAL(hhogsNumChanged(const HWTeam&)),
 		hwnet, SLOT(onHedgehogsNumChanged(const HWTeam&)));
@@ -412,17 +490,19 @@ void HWForm::_NetConnect(const QString & hostName, quint16 port, const QString &
 	connect(ui.pageNetGame->pGameCFG, SIGNAL(seedChanged(const QString &)), hwnet, SLOT(onSeedChanged(const QString &)));
 	connect(ui.pageNetGame->pGameCFG, SIGNAL(mapChanged(const QString &)), hwnet, SLOT(onMapChanged(const QString &)));
 	connect(ui.pageNetGame->pGameCFG, SIGNAL(themeChanged(const QString &)), hwnet, SLOT(onThemeChanged(const QString &)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(initHealthChanged(quint32)), hwnet, SLOT(onInitHealthChanged(quint32)));
-	connect(ui.pageNetGame->pGameCFG, SIGNAL(turnTimeChanged(quint32)), hwnet, SLOT(onTurnTimeChanged(quint32)));
+	connect(ui.pageNetGame->pGameCFG, SIGNAL(initHealthChanged(int)), hwnet, SLOT(onInitHealthChanged(int)));
+	connect(ui.pageNetGame->pGameCFG, SIGNAL(turnTimeChanged(int)), hwnet, SLOT(onTurnTimeChanged(int)));
 	connect(ui.pageNetGame->pGameCFG, SIGNAL(fortsModeChanged(bool)), hwnet, SLOT(onFortsModeChanged(bool)));
+	connect(ui.pageNetGame->pGameCFG, SIGNAL(teamsDivideChanged(bool)), hwnet, SLOT(onTeamsDivideChanged(bool)));
 
 	connect(hwnet, SIGNAL(Disconnected()), this, SLOT(ForcedDisconnect()));
 	connect(hwnet, SIGNAL(seedChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setSeed(const QString &)));
 	connect(hwnet, SIGNAL(mapChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setMap(const QString &)));
 	connect(hwnet, SIGNAL(themeChanged(const QString &)), ui.pageNetGame->pGameCFG, SLOT(setTheme(const QString &)));
-	connect(hwnet, SIGNAL(initHealthChanged(quint32)), ui.pageNetGame->pGameCFG, SLOT(setInitHealth(quint32)));
-	connect(hwnet, SIGNAL(turnTimeChanged(quint32)), ui.pageNetGame->pGameCFG, SLOT(setTurnTime(quint32)));
+	connect(hwnet, SIGNAL(initHealthChanged(int)), ui.pageNetGame->pGameCFG, SLOT(setInitHealth(int)));
+	connect(hwnet, SIGNAL(turnTimeChanged(int)), ui.pageNetGame->pGameCFG, SLOT(setTurnTime(int)));
 	connect(hwnet, SIGNAL(fortsModeChanged(bool)), ui.pageNetGame->pGameCFG, SLOT(setFortsMode(bool)));
+	connect(hwnet, SIGNAL(teamsDivideChanged(bool)), ui.pageNetGame->pGameCFG, SLOT(setTeamsDivide(bool)));
 	connect(hwnet, SIGNAL(hhnumChanged(const HWTeam&)),
 		ui.pageNetGame->pNetTeamsWidget, SLOT(changeHHNum(const HWTeam&)));
 	connect(hwnet, SIGNAL(teamColorChanged(const HWTeam&)),
@@ -444,7 +524,7 @@ void HWForm::NetConnect()
 		delete netHost;
 		netHost = new QString(hpd->leHost->text());
 		netPort = hpd->sbPort->value();
-		_NetConnect(*netHost, netPort, ui.pageOptions->editNetNick->text());
+		NetConnectServer(*netHost, netPort);
 	}
 }
 
@@ -462,12 +542,16 @@ void HWForm::NetStartServer()
 		return;
 	}
 
-	_NetConnect("localhost", pnetserver->getRunningPort(), ui.pageOptions->editNetNick->text());
+	QTimer::singleShot(250, this, SLOT(AsyncNetServerStart()));
 
-	if (ui.pageNet->rbLocalGame->isChecked())
-		pRegisterServer = new HWNetUdpServer(0, ui.pageNetServer->leServerDescr->text(), ui.pageNetServer->sbPort->value());
-	else
-		pRegisterServer = new HWNetWwwServer(0, ui.pageNetServer->leServerDescr->text(), ui.pageNetServer->sbPort->value());
+	pRegisterServer = new HWNetUdpServer(0,
+			ui.pageNetServer->leServerDescr->text(),
+			ui.pageNetServer->sbPort->value());
+}
+
+void HWForm::AsyncNetServerStart()
+{
+	NetConnectServer("localhost", pnetserver->getRunningPort());
 }
 
 void HWForm::NetDisconnect()
@@ -475,7 +559,7 @@ void HWForm::NetDisconnect()
   if(hwnet) {
     hwnet->Disconnect();
     delete hwnet;
-    hwnet=0;
+    hwnet = 0;
   }
   if(pnetserver) {
     if (pRegisterServer)
@@ -486,42 +570,40 @@ void HWForm::NetDisconnect()
 
     pnetserver->StopServer();
     delete pnetserver;
-    pnetserver=0;
+    pnetserver = 0;
   }
 }
 
 void HWForm::ForcedDisconnect()
 {
-  if(pnetserver) return; // we have server - let it care of all things
-  if (hwnet) {
-    hwnet->deleteLater();
-    hwnet=0;
-    QMessageBox::warning(this, QMessageBox::tr("Network"),
-			 QMessageBox::tr("Connection to server is lost"));
-  }
-  GoBack();
+	if(pnetserver) return; // we have server - let it care of all things
+	if (hwnet) {
+		hwnet->deleteLater();
+		hwnet = 0;
+		QMessageBox::warning(this, QMessageBox::tr("Network"),
+				QMessageBox::tr("Connection to server is lost"));
+	}
+	GoBack();
+}
+
+void HWForm::NetConnected()
+{
+	GoToPage(ID_PAGE_ROOMSLIST);
 }
 
 void HWForm::NetGameEnter()
 {
-	GoToPage(ID_PAGE_NETCFG);
-}
-
-void HWForm::NetStartGame()
-{
-  ui.pageNetGame->BtnGo->setText(QPushButton::tr("Waiting"));
-  ui.pageNetGame->BtnGo->setEnabled(false);
-  hwnet->StartGame();
+	GoToPage(ID_PAGE_NETGAME);
 }
 
 void HWForm::AddNetTeam(const HWTeam& team)
 {
-  ui.pageNetGame->pNetTeamsWidget->addTeam(team);
+	ui.pageNetGame->pNetTeamsWidget->addTeam(team);
 }
 
 void HWForm::StartMPGame()
 {
-	QString ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageMultiplayer->gameCFG->WeaponsName->currentText());
+	QString ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageMultiplayer->gameCFG->WeaponsName->currentText());
 
 	CreateGame(ui.pageMultiplayer->gameCFG, ui.pageMultiplayer->teamsSelect, ammo);
 
@@ -532,6 +614,8 @@ void HWForm::GameStateChanged(GameState gameState)
 {
 	switch(gameState) {
 		case gsStarted: {
+			Music(false);
+			GoToPage(ID_PAGE_INGAME);
 			ui.pageGameStats->labelGameStats->setText("");
 			if (pRegisterServer)
 			{
@@ -542,20 +626,20 @@ void HWForm::GameStateChanged(GameState gameState)
 		}
 		case gsFinished: {
 			GoBack();
+			Music(ui.pageOptions->CBEnableMusic->isChecked());
 			GoToPage(ID_PAGE_GAMESTATS);
+			if (hwnet) hwnet->gameFinished();
 			break;
 		}
 		default: {
 			quint8 id = ui.Pages->currentIndex();
-			if (id == ID_PAGE_INGAME) GoBack();
+			if (id == ID_PAGE_INGAME) {
+				GoBack();
+				Music(ui.pageOptions->CBEnableMusic->isChecked());
+				if (hwnet) hwnet->gameFinished();
+			}
 		};
 	}
-
-	if (hwnet)
-	{
-		ui.pageNetGame->BtnGo->setText(QPushButton::tr("Go!"));
-		ui.pageNetGame->BtnGo->setEnabled(true);
-	}
 }
 
 void HWForm::AddStatText(const QString & msg)
@@ -600,8 +684,6 @@ void HWForm::CreateGame(GameCFGWidget * gamecfg, TeamSelWidget* pTeamSelWidget,
 	connect(game, SIGNAL(GameStats(char, const QString &)), this, SLOT(GameStats(char, const QString &)));
 	connect(game, SIGNAL(ErrorMessage(const QString &)), this, SLOT(ShowErrorMessage(const QString &)), Qt::QueuedConnection);
 	connect(game, SIGNAL(HaveRecord(bool, const QByteArray &)), this, SLOT(GetRecord(bool, const QByteArray &)));
-
-	GoToPage(ID_PAGE_INGAME);
 }
 
 void HWForm::ShowErrorMessage(const QString & msg)
@@ -649,15 +731,18 @@ void HWForm::StartTraining()
 void HWForm::CreateNetGame()
 {
 	QString ammo;
-	if (pnetserver) {
-	  ammo=ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
+	if (hwnet->isRoomChief()) {
+		ammo = ui.pageSelectWeapon->pWeapons->getWeaponsString(ui.pageNetGame->pGameCFG->WeaponsName->currentText());
 	} else {
-	  ammo=ui.pageNetGame->pGameCFG->getNetAmmo();
+		ammo = ui.pageNetGame->pGameCFG->getNetAmmo();
 	}
+	
 	CreateGame(ui.pageNetGame->pGameCFG, ui.pageNetGame->pNetTeamsWidget, ammo);
 
 	connect(game, SIGNAL(SendNet(const QByteArray &)), hwnet, SLOT(SendNet(const QByteArray &)));
+	connect(game, SIGNAL(SendChat(const QString &)), hwnet, SLOT(chatLineToNet(const QString &)));
 	connect(hwnet, SIGNAL(FromNet(const QByteArray &)), game, SLOT(FromNet(const QByteArray &)));
+	connect(hwnet, SIGNAL(chatStringFromNet(const QString &)), game, SLOT(FromNetChat(const QString &)));
 
 	game->StartNet();
 }
@@ -667,3 +752,30 @@ void HWForm::closeEvent(QCloseEvent *event)
 	config->SaveOptions();
 	event->accept();
 }
+
+void HWForm::Music(bool checked)
+{
+	if (checked)
+		sdli.StartMusic();
+	else
+		sdli.StopMusic();
+}
+
+void HWForm::NetGameMaster()
+{
+	ui.pageNetGame->BtnMaster->setVisible(true);
+	ui.pageNetGame->restrictJoins->setChecked(false);
+	ui.pageNetGame->restrictTeamAdds->setChecked(false);
+	
+	if (hwnet)
+	{
+		connect(ui.pageNetGame->startGame, SIGNAL(triggered()), hwnet, SLOT(startGame()));
+		connect(ui.pageNetGame->restrictJoins, SIGNAL(triggered()), hwnet, SLOT(toggleRestrictJoins()));
+		connect(ui.pageNetGame->restrictTeamAdds, SIGNAL(triggered()), hwnet, SLOT(toggleRestrictTeamAdds()));
+	}
+}
+
+void HWForm::NetGameSlave()
+{
+	ui.pageNetGame->BtnMaster->setVisible(false);
+}
